@@ -58,13 +58,13 @@ program packmol
   double precision, allocatable :: x(:), xprint(:) ! (nn)
   double precision :: v1(3),v2(3),v3(3)
   double precision :: radscale, value
-  double precision :: cmx, cmy, cmz, beta, gama, teta
-  double precision :: xtemp, ytemp, ztemp
+  double precision :: xcm(3), beta, gama, teta
+  double precision :: xtemp(3)
   double precision :: fx, bestf, flast, fprint, all_type_fx
   double precision :: fimp, fimprov
   double precision, parameter :: pi=4.d0*datan(1.d0)
 
-  real :: etime, tarray(2), time0
+  real :: etime, tarray(2), time0, time_writeout
   
   character(len=strl) :: record, restart_from_temp, restart_to_temp
   character(len=strl) :: xyzfile
@@ -92,6 +92,7 @@ program packmol
   ! Start time computation
 
   time0 = etime(tarray)
+  time_writeout = time0
 
   ! Reading input file
 
@@ -107,18 +108,13 @@ program packmol
 
   ! Put fixed molecules in the specified position
 
-  do itype = 1, ntype
-    fixed(itype) = .false.
-  end do
-
+  fixed(1:ntype) = .false.
   do irest = 1, nrest
     if(ityperest(irest).eq.1) then
       do itype = 1, ntype
         if(irestline(irest).gt.linestrut(itype,1).and.&
            irestline(irest).lt.linestrut(itype,2)) then
-          cmx = restpars(irest,1) 
-          cmy = restpars(irest,2)
-          cmz = restpars(irest,3)    
+          xcm = restpars(irest,1:3)
           beta = restpars(irest,4) 
           gama = restpars(irest,5) 
           teta = restpars(irest,6) 
@@ -130,18 +126,16 @@ program packmol
           idatom = idfirst(itype) - 1
           do iatom = 1, natoms(itype)
             idatom = idatom + 1
-            xtemp =   coor(idatom,1)*v1(1) &
-                    + coor(idatom,2)*v2(1) &
-                    + coor(idatom,3)*v3(1) 
-            ytemp =   coor(idatom,1)*v1(2) &
-                    + coor(idatom,2)*v2(2) &
-                    + coor(idatom,3)*v3(2) 
-            ztemp =   coor(idatom,1)*v1(3) &
-                    + coor(idatom,2)*v2(3) &
-                    + coor(idatom,3)*v3(3) 
-            coor(idatom, 1) = xtemp + cmx
-            coor(idatom, 2) = ytemp + cmy
-            coor(idatom, 3) = ztemp + cmz 
+            xtemp(1) = coor(idatom,1)*v1(1) &
+                     + coor(idatom,2)*v2(1) &
+                     + coor(idatom,3)*v3(1) 
+            xtemp(2) = coor(idatom,1)*v1(2) &
+                     + coor(idatom,2)*v2(2) &
+                     + coor(idatom,3)*v3(2) 
+            xtemp(3) = coor(idatom,1)*v1(3) &
+                     + coor(idatom,2)*v2(3) &
+                     + coor(idatom,3)*v3(3) 
+            coor(idatom, 1:3) = xtemp + xcm
           end do
           record = name(itype)
           write(*,*) ' Molecule ',trim(adjustl(record)),'(',itype,') will be fixed.' 
@@ -181,11 +175,11 @@ program packmol
       fixedoninput(itype) = .false.
     end if
   end do
-  ntfix = ntype
+  ntype_with_fixed = ntype
   ntype = ntemp     
 
-  do i = 1, ntfix - ntype 
-    do itype = 1, ntfix - 1
+  do i = 1, ntype_with_fixed - ntype 
+    do itype = 1, ntype_with_fixed - 1
       if(fixed(itype)) then
         record = name(itype)
         restart_to_temp = restart_to(itype)
@@ -251,11 +245,11 @@ program packmol
   !
   ! ntype: 1...ntype (counter for the number of free structures)
   !
-  ! ntfix: 1...ntype...ntfix (counter for the total number of structures)
+  ! ntype_with_fixed: 1...ntype...ntype_with_fixed (counter for the total number of structures)
   !
 
   ntmol = 0
-  do itype = 1, ntfix
+  do itype = 1, ntype_with_fixed
     ntmol = ntmol + nmols(itype)
   end do
   ntotmol = 0 
@@ -270,13 +264,281 @@ program packmol
 
   ! Computing the total number of fixed atoms
 
-  natfix = 0
+  nfixedat = 0
   if(fix) then
-    do iftype = ntype + 1, ntfix
-      natfix = natfix + natoms(iftype)
+    do iftype = ntype + 1, ntype_with_fixed
+      nfixedat = nfixedat + natoms(iftype)
     end do
   end if       
-  write(*,*) ' Total number of fixed atoms: ', natfix
+  write(*,*) ' Total number of fixed atoms: ', nfixedat
+
+  ! Setting the vector that contains the default tolerances
+
+  do i = 1, ntotat
+    radius(i) = dism/2.d0
+    fscale(i) = 1.d0
+    if ( use_short_tol ) then
+      use_short_radius(i) = .true.
+    else
+      use_short_radius(i) = .false.
+    end if
+    short_radius(i) = short_tol_dist/2.d0
+    short_radius_scale(i) = short_tol_scale
+  end do
+
+  ! Setting the radius defined for atoms of each molecule, 
+  ! but not atom-specific, first
+
+  icart = 0
+  do itype = 1, ntype_with_fixed
+    iline = linestrut(itype,1)
+    iline_atoms = 0 
+    do while( iline <= linestrut(itype,2) )
+      if ( keyword(iline,1) == "atoms" ) then
+        iline_atoms = iline
+        iline = iline + 1
+        cycle
+      end if
+      if ( keyword(iline,1) == "end" .and. &    
+           keyword(iline,2) == "atoms" ) then
+        iline_atoms = 0  
+        iline = iline + 1
+        cycle
+      end if
+      if ( iline_atoms == 0 ) then
+        !
+        ! Read radius
+        !
+        if ( keyword(iline,1) == "radius" ) then
+          read(keyword(iline,2),*,iostat=ioerr) value
+          if ( ioerr /= 0 ) then
+            write(*,*) ' ERROR: Could not read radius from keyword. '
+            stop exit_code_input_error
+          end if
+          iicart = icart
+          do imol = 1, nmols(itype)
+            do iatom = 1, natoms(itype)
+              iicart = iicart + 1
+              radius(iicart) = value
+            end do
+          end do
+        end if
+        !
+        ! Read minimum-distance function scale
+        !
+        if ( keyword(iline,1) == "fscale" ) then
+          read(keyword(iline,2),*,iostat=ioerr) value
+          if ( ioerr /= 0 ) then
+            write(*,*) ' ERROR: Could not read fscale value from keyword. '
+            stop exit_code_input_error
+          end if
+          iicart = icart
+          do imol = 1, nmols(itype)
+            do iatom = 1, natoms(itype)
+              iicart = iicart + 1
+              fscale(iicart) = value
+            end do
+          end do
+        end if
+        !
+        ! Read short_radius
+        !  
+        if ( keyword(iline,1) == "short_radius" ) then
+          read(keyword(iline,2),*,iostat=ioerr) value
+          if ( ioerr /= 0 ) then
+            write(*,*) ' ERROR: Could not read short_radius value from keyword. '
+            stop exit_code_input_error
+          end if
+          iicart = icart
+          do imol = 1, nmols(itype)
+            do iatom = 1, natoms(itype)
+              iicart = iicart + 1
+              short_radius(iicart) = value
+              use_short_radius(iicart) = .true.
+            end do
+          end do
+        end if
+        !
+        ! Read short_radius scale
+        !  
+        if ( keyword(iline,1) == "short_radius_scale" ) then
+          read(keyword(iline,2),*,iostat=ioerr) value
+          if ( ioerr /= 0 ) then
+            write(*,*) ' ERROR: Could not read short_radius_scale value from keyword. '
+            stop exit_code_input_error
+          end if
+          iicart = icart
+          do imol = 1, nmols(itype)
+            do iatom = 1, natoms(itype)
+              iicart = iicart + 1
+              short_radius_scale(iicart) = value
+              use_short_radius(iicart) = .true.
+            end do
+          end do
+        end if
+      end if
+      iline = iline + 1
+    end do
+    icart = icart + nmols(itype)*natoms(itype)
+  end do
+
+  ! If some radius was defined using atom-specific definitions, overwrite
+  ! the general radius defined for the molecule
+
+  icart = 0
+  do itype = 1, ntype_with_fixed
+    iline = linestrut(itype,1)
+    iline_atoms = 0 
+    do while( iline <= linestrut(itype,2) )
+      if ( keyword(iline,1) == "atoms" ) then
+        iline_atoms = iline
+        iline = iline + 1
+        cycle
+      end if
+      if ( keyword(iline,1) == "end" .and. &    
+           keyword(iline,2) == "atoms" ) then
+        iline_atoms = 0  
+        iline = iline + 1
+        cycle
+      end if
+      if ( iline_atoms /= 0 ) then
+        !
+        ! Read atom specific radius
+        !
+        if ( keyword(iline,1) == "radius" ) then
+          read(keyword(iline,2),*,iostat=ioerr) value
+          if ( ioerr /= 0 ) then
+            write(*,*) ' ERROR: Could not read radius from keyword. '
+            stop exit_code_input_error
+          end if
+          ival = 2
+          do
+            read(keyword(iline_atoms,ival),*,iostat=ioerr) iat
+            if ( ioerr /= 0 ) exit
+            if ( iat > natoms(itype) ) then
+              write(*,*) ' ERROR: atom selection with index greater than number of '
+              write(*,*) '        atoms in structure ', itype
+              stop exit_code_input_error
+            end if
+            radius(icart+iat) = value
+            ival = ival + 1
+          end do
+        end if
+        !
+        ! Read atom specific function scale
+        !
+        if ( keyword(iline,1) == "fscale" ) then
+          read(keyword(iline,2),*,iostat=ioerr) value
+          if ( ioerr /= 0 ) then
+            write(*,*) ' ERROR: Could not read fscale value from keyword. '
+            stop exit_code_input_error
+          end if
+          ival = 2
+          do
+            read(keyword(iline_atoms,ival),*,iostat=ioerr) iat
+            if ( ioerr /= 0 ) exit
+            if ( iat > natoms(itype) ) then
+              write(*,*) ' ERROR: atom selection with index greater than number of '
+              write(*,*) '        atoms in structure ', itype
+              stop exit_code_input_error
+            end if
+            fscale(icart+iat) = value
+            ival = ival + 1
+          end do
+        end if
+        !
+        ! Read atom specific short radius
+        !
+        if ( keyword(iline,1) == "short_radius" ) then
+          read(keyword(iline,2),*,iostat=ioerr) value
+          if ( ioerr /= 0 ) then
+            write(*,*) ' ERROR: Could not read short_radius value from keyword. '
+            stop exit_code_input_error
+          end if
+          ival = 2
+          do
+            read(keyword(iline_atoms,ival),*,iostat=ioerr) iat
+            if ( ioerr /= 0 ) exit
+            if ( iat > natoms(itype) ) then
+              write(*,*) ' ERROR: atom selection with index greater than number of '
+              write(*,*) '        atoms in structure ', itype
+              stop exit_code_input_error
+            end if
+            short_radius(icart+iat) = value
+            use_short_radius(icart+iat) = .true.
+            ival = ival + 1
+          end do
+        end if
+        !
+        ! Read atom specific short radius function scale
+        !
+        if ( keyword(iline,1) == "short_radius_scale" ) then
+          read(keyword(iline,2),*,iostat=ioerr) value
+          if ( ioerr /= 0 ) then
+            write(*,*) ' ERROR: Could not read short_radius_scale value from keyword. '
+            stop exit_code_input_error
+          end if
+          ival = 2
+          do
+            read(keyword(iline_atoms,ival),*,iostat=ioerr) iat
+            if ( ioerr /= 0 ) exit
+            if ( iat > natoms(itype) ) then
+              write(*,*) ' ERROR: atom selection with index greater than number of '
+              write(*,*) '        atoms in structure ', itype
+              stop exit_code_input_error
+            end if
+            short_radius_scale(icart+iat) = value
+            use_short_radius(icart+iat) = .true.
+            ival = ival + 1
+          end do
+        end if
+      end if
+      iline = iline + 1
+    end do
+    iicart = icart
+    icart = icart + natoms(itype)
+    do imol = 2, nmols(itype)
+      do iatom = 1, natoms(itype)
+        icart = icart + 1
+        radius(icart) = radius(iicart+iatom)
+        fscale(icart) = fscale(iicart+iatom)
+        short_radius(icart) = short_radius(iicart+iatom)
+        short_radius_scale(icart) = short_radius_scale(iicart+iatom)
+        use_short_radius(icart) = use_short_radius(iicart+iatom)
+      end do
+    end do
+  end do
+
+  ! Check if the short radii were set correctly, if the case
+
+  ioerr = 0
+  do i = 1, ntotat
+    if ( use_short_radius(i) ) then
+     if ( short_radius(i) >= radius(i) ) then 
+       write(*,*) ' ERROR: The short radius must be smaller than the default radius. '
+       write(*,*) '        (the default radius is one half of the default tolerance).'
+       stop exit_code_input_error
+     end if
+   end if
+  end do
+
+  ! If using PBC, add the extra constraint
+
+  radmax = 0.d0
+  do i = 1, ntotat
+    radmax = dmax1(radmax,2.d0*radius(i))
+  end do
+
+  if (using_pbc) then
+    nrest = nrest + 1
+    irestline(nrest) = -1
+    ityperest(nrest) = 3
+    restpars(nrest,1:3) = pbc_min
+    restpars(nrest,4:6) = pbc_max
+    write(*,*) " PBC on: Constraint automatically added to non-fixed atoms:"
+    write(*,"(a, 6f8.2)") "  -> inside box ", restpars(nrest,1:6)
+    write(*,*) ' Updated total number of restrictions: ', nrest
+  end if
 
   ! Setting the array that contains the restrictions per atom
 
@@ -423,256 +685,6 @@ program packmol
     end do
   end do
  
-  ! Setting the vector that contains the default tolerances
-
-  do i = 1, ntotat
-    radius(i) = dism/2.d0
-    fscale(i) = 1.d0
-    if ( use_short_tol ) then
-      use_short_radius(i) = .true.
-    else
-      use_short_radius(i) = .false.
-    end if
-    short_radius(i) = short_tol_dist/2.d0
-    short_radius_scale(i) = short_tol_scale
-  end do
-
-  ! Setting the radius defined for atoms of each molecule, 
-  ! but not atom-specific, first
-
-  icart = 0
-  do itype = 1, ntfix
-    iline = linestrut(itype,1)
-    iline_atoms = 0 
-    do while( iline <= linestrut(itype,2) )
-      if ( keyword(iline,1) == "atoms" ) then
-        iline_atoms = iline
-        iline = iline + 1
-        cycle
-      end if
-      if ( keyword(iline,1) == "end" .and. &    
-           keyword(iline,2) == "atoms" ) then
-        iline_atoms = 0  
-        iline = iline + 1
-        cycle
-      end if
-      if ( iline_atoms == 0 ) then
-        !
-        ! Read radius
-        !
-        if ( keyword(iline,1) == "radius" ) then
-          read(keyword(iline,2),*,iostat=ioerr) value
-          if ( ioerr /= 0 ) then
-            write(*,*) ' ERROR: Could not read radius from keyword. '
-            stop exit_code_input_error
-          end if
-          iicart = icart
-          do imol = 1, nmols(itype)
-            do iatom = 1, natoms(itype)
-              iicart = iicart + 1
-              radius(iicart) = value
-            end do
-          end do
-        end if
-        !
-        ! Read minimum-distance function scale
-        !
-        if ( keyword(iline,1) == "fscale" ) then
-          read(keyword(iline,2),*,iostat=ioerr) value
-          if ( ioerr /= 0 ) then
-            write(*,*) ' ERROR: Could not read fscale value from keyword. '
-            stop exit_code_input_error
-          end if
-          iicart = icart
-          do imol = 1, nmols(itype)
-            do iatom = 1, natoms(itype)
-              iicart = iicart + 1
-              fscale(iicart) = value
-            end do
-          end do
-        end if
-        !
-        ! Read short_radius
-        !  
-        if ( keyword(iline,1) == "short_radius" ) then
-          read(keyword(iline,2),*,iostat=ioerr) value
-          if ( ioerr /= 0 ) then
-            write(*,*) ' ERROR: Could not read short_radius value from keyword. '
-            stop exit_code_input_error
-          end if
-          iicart = icart
-          do imol = 1, nmols(itype)
-            do iatom = 1, natoms(itype)
-              iicart = iicart + 1
-              short_radius(iicart) = value
-              use_short_radius(iicart) = .true.
-            end do
-          end do
-        end if
-        !
-        ! Read short_radius scale
-        !  
-        if ( keyword(iline,1) == "short_radius_scale" ) then
-          read(keyword(iline,2),*,iostat=ioerr) value
-          if ( ioerr /= 0 ) then
-            write(*,*) ' ERROR: Could not read short_radius_scale value from keyword. '
-            stop exit_code_input_error
-          end if
-          iicart = icart
-          do imol = 1, nmols(itype)
-            do iatom = 1, natoms(itype)
-              iicart = iicart + 1
-              short_radius_scale(iicart) = value
-              use_short_radius(iicart) = .true.
-            end do
-          end do
-        end if
-      end if
-      iline = iline + 1
-    end do
-    icart = icart + nmols(itype)*natoms(itype)
-  end do
-
-  ! If some radius was defined using atom-specific definitions, overwrite
-  ! the general radius defined for the molecule
-
-  icart = 0
-  do itype = 1, ntfix
-    iline = linestrut(itype,1)
-    iline_atoms = 0 
-    do while( iline <= linestrut(itype,2) )
-      if ( keyword(iline,1) == "atoms" ) then
-        iline_atoms = iline
-        iline = iline + 1
-        cycle
-      end if
-      if ( keyword(iline,1) == "end" .and. &    
-           keyword(iline,2) == "atoms" ) then
-        iline_atoms = 0  
-        iline = iline + 1
-        cycle
-      end if
-      if ( iline_atoms /= 0 ) then
-        !
-        ! Read atom specific radius
-        !
-        if ( keyword(iline,1) == "radius" ) then
-          read(keyword(iline,2),*,iostat=ioerr) value
-          if ( ioerr /= 0 ) then
-            write(*,*) ' ERROR: Could not read radius from keyword. '
-            stop exit_code_input_error
-          end if
-          ival = 2
-          do
-            read(keyword(iline_atoms,ival),*,iostat=ioerr) iat
-            if ( ioerr /= 0 ) exit
-            if ( iat > natoms(itype) ) then
-              write(*,*) ' ERROR: atom selection with index greater than number of '
-              write(*,*) '        atoms in structure ', itype
-              stop exit_code_input_error
-            end if
-            radius(icart+iat) = value
-            ival = ival + 1
-          end do
-        end if
-        !
-        ! Read atom specific function scale
-        !
-        if ( keyword(iline,1) == "fscale" ) then
-          read(keyword(iline,2),*,iostat=ioerr) value
-          if ( ioerr /= 0 ) then
-            write(*,*) ' ERROR: Could not read fscale value from keyword. '
-            stop exit_code_input_error
-          end if
-          ival = 2
-          do
-            read(keyword(iline_atoms,ival),*,iostat=ioerr) iat
-            if ( ioerr /= 0 ) exit
-            if ( iat > natoms(itype) ) then
-              write(*,*) ' ERROR: atom selection with index greater than number of '
-              write(*,*) '        atoms in structure ', itype
-              stop exit_code_input_error
-            end if
-            fscale(icart+iat) = value
-            ival = ival + 1
-          end do
-        end if
-        !
-        ! Read atom specific short radius
-        !
-        if ( keyword(iline,1) == "short_radius" ) then
-          read(keyword(iline,2),*,iostat=ioerr) value
-          if ( ioerr /= 0 ) then
-            write(*,*) ' ERROR: Could not read short_radius value from keyword. '
-            stop exit_code_input_error
-          end if
-          ival = 2
-          do
-            read(keyword(iline_atoms,ival),*,iostat=ioerr) iat
-            if ( ioerr /= 0 ) exit
-            if ( iat > natoms(itype) ) then
-              write(*,*) ' ERROR: atom selection with index greater than number of '
-              write(*,*) '        atoms in structure ', itype
-              stop exit_code_input_error
-            end if
-            short_radius(icart+iat) = value
-            use_short_radius(icart+iat) = .true.
-            ival = ival + 1
-          end do
-        end if
-        !
-        ! Read atom specific short radius function scale
-        !
-        if ( keyword(iline,1) == "short_radius_scale" ) then
-          read(keyword(iline,2),*,iostat=ioerr) value
-          if ( ioerr /= 0 ) then
-            write(*,*) ' ERROR: Could not read short_radius_scale value from keyword. '
-            stop exit_code_input_error
-          end if
-          ival = 2
-          do
-            read(keyword(iline_atoms,ival),*,iostat=ioerr) iat
-            if ( ioerr /= 0 ) exit
-            if ( iat > natoms(itype) ) then
-              write(*,*) ' ERROR: atom selection with index greater than number of '
-              write(*,*) '        atoms in structure ', itype
-              stop exit_code_input_error
-            end if
-            short_radius_scale(icart+iat) = value
-            use_short_radius(icart+iat) = .true.
-            ival = ival + 1
-          end do
-        end if
-      end if
-      iline = iline + 1
-    end do
-    iicart = icart
-    icart = icart + natoms(itype)
-    do imol = 2, nmols(itype)
-      do iatom = 1, natoms(itype)
-        icart = icart + 1
-        radius(icart) = radius(iicart+iatom)
-        fscale(icart) = fscale(iicart+iatom)
-        short_radius(icart) = short_radius(iicart+iatom)
-        short_radius_scale(icart) = short_radius_scale(iicart+iatom)
-        use_short_radius(icart) = use_short_radius(iicart+iatom)
-      end do
-    end do
-  end do
-
-  ! Check if the short radii were set correctly, if the case
-
-  ioerr = 0
-  do i = 1, ntotat
-    if ( use_short_radius(i) ) then
-     if ( short_radius(i) >= radius(i) ) then 
-       write(*,*) ' ERROR: The short radius must be smaller than the default radius. '
-       write(*,*) '        (the default radius is one half of the default tolerance).'
-       stop exit_code_input_error
-     end if
-   end if
-  end do
-
   ! If there are no variables (only fixed molecules, stop)
 
   if(n.eq.0) then
@@ -795,7 +807,7 @@ program packmol
         loop = loop + 1
 
         ! Moving bad molecules
-        if(radscale == 1.d0 .and. fimp.le.10.d0) then
+        if(.not. disable_movebad .and. radscale == 1.d0 .and. fimp.le.10.d0) then
           movebadprint = .true.
           call movebad(n,x,fx,movebadprint)
           flast = fx
@@ -889,20 +901,22 @@ program packmol
         write(*,dash3_line)
 
         ! If this is the best structure so far
-        if( mod(loop+1,writeout) == 0 .and. all_type_fx < fprint ) then
-          call output(n,x,xyzout)
-          write(*,*) ' Current solution written to file: ', trim(adjustl(xyzout))
-          if ( crd ) write(*,*) ' ... and to CRD file: ', trim(adjustl(crdfile))
-          fprint = all_type_fx
-          do i = 1, n
-            xprint(i) = x(i)
-          end do
-
-        ! If the user required printing even bad structures
-        else if ( mod(loop+1,writeout) == 0 .and. writebad ) then
-          call output(n,x,xyzout)
-          write(*,*) ' Writing current (perhaps bad) structure to file: ', trim(adjustl(xyzout))
-          if ( crd ) write(*,*) ' ... and to CRD file: ', trim(adjustl(crdfile))
+        if(etime(tarray) - time_writeout > writeout) then
+          time_writeout = etime(tarray)
+          if(all_type_fx < fprint ) then
+            call output(n,x,xyzout)
+            write(*,*) ' Current solution written to file: ', trim(adjustl(xyzout))
+            if ( crd ) write(*,*) ' ... and to CRD file: ', trim(adjustl(crdfile))
+            fprint = all_type_fx
+            do i = 1, n
+              xprint(i) = x(i)
+            end do
+          ! If the user required printing even bad structures
+          else if (writebad) then
+            call output(n,x,xyzout)
+            write(*,*) ' Writing current (perhaps bad) structure to file: ', trim(adjustl(xyzout))
+            if ( crd ) write(*,*) ' ... and to CRD file: ', trim(adjustl(crdfile))
+          end if
         end if
 
         ! Restore vector for packing this type of molecule, if the case
