@@ -16,7 +16,7 @@ subroutine getinp()
    use pbc
 
    implicit none
-   integer :: i, k, ii, iarg, iline, idatom, iatom, in, lixo, irest, itype, itest,&
+   integer :: i, k, ii, iarg, iline, idatom, iatom, in, irest, itype, itest,&
       imark, ioerr, nloop0, iread, idfirstatom
    double precision :: clen
    character(len=strl) :: record, blank
@@ -43,6 +43,7 @@ subroutine getinp()
    nloop0 = 0
    movefrac = 0.05
    movebadrandom = .false.
+   disable_movebad = .false.
    precision = 1.d-2
    writebad = .false.
    add_amber_ter = .false.
@@ -55,7 +56,7 @@ subroutine getinp()
    packall = .false.
    use_short_tol = .false.
    crd = .false.
-
+   hexadecimal_indices = .false.
    inside_structure = .false.
 
    do i = 1, nlines
@@ -85,12 +86,17 @@ subroutine getinp()
       else if(keyword(i,1).eq.'movebadrandom') then
          movebadrandom = .true.
          write(*,*) ' Will move randomly bad molecues (movebadrandom) '
+      else if(keyword(i,1).eq.'disable_movebad') then
+         disable_movebad = .true.
+         write(*,*) ' Move-bad heursitic is dsabled. '
       else if(keyword(i,1).eq.'chkgrad') then
          chkgrad = .true.
       else if(keyword(i,1).eq.'writeout') then
          read(keyword(i,2),*,iostat=ioerr) writeout
          if ( ioerr /= 0 ) exit
          write(*,*) ' Output frequency: ', writeout
+      else if(keyword(i,1).eq.'hexadecimal_indices') then
+         hexadecimal_indices = .true.
       else if(keyword(i,1).eq.'maxit') then
          read(keyword(i,2),*,iostat=ioerr) maxit
          if ( ioerr /= 0 ) exit
@@ -159,30 +165,37 @@ subroutine getinp()
          if ( ioerr /= 0 ) exit
          write(*,*) ' Optional printvalue 2 set: ', iprint2
       else if(keyword(i,1).eq.'pbc') then
-         read(keyword(i,2),*,iostat=ioerr) pbc_box(1)
-         read(keyword(i,3),*,iostat=ioerr) pbc_box(2)
-         read(keyword(i,4),*,iostat=ioerr) pbc_box(3)
+         read(keyword(i,2),*,iostat=ioerr) pbc_min(1)
+         read(keyword(i,3),*,iostat=ioerr) pbc_min(2)
+         read(keyword(i,4),*,iostat=ioerr) pbc_min(3)
          if ( ioerr /= 0 ) exit
-         if (maxkeywords > 4 .and. trim(adjustl(keyword(i,5))) /= "none") then 
-            read(keyword(i,5),*,iostat=ioerr) pbc_box(4)
-            read(keyword(i,6),*,iostat=ioerr) pbc_box(5)
-            read(keyword(i,7),*,iostat=ioerr) pbc_box(6)
-            if ( ioerr /= 0 ) exit
+         if (maxkeywords > 4)  then
+            if (trim(adjustl(keyword(i,5))) /= "none") then 
+               read(keyword(i,5),*,iostat=ioerr) pbc_max(1)
+               read(keyword(i,6),*,iostat=ioerr) pbc_max(2)
+               read(keyword(i,7),*,iostat=ioerr) pbc_max(3)
+               if ( ioerr /= 0 ) exit
+            else
+               pbc_max = pbc_min
+               pbc_min = 0.0
+            end if
          else 
-            pbc_box(4:6) = pbc_box(1:3)
-            pbc_box(1:3) = 0.0
+            pbc_max = pbc_min
+            pbc_min = 0.0
          end if
          if ( ioerr /= 0 ) exit
          using_pbc = .true.
-         pbc_length(1:3) = pbc_box(4:6) - pbc_box(1:3)
-         if ( any(pbc_length(1:3) <= 0.0) ) then
-            write(*,*) ' ERROR: Length of PBC box must be positive in all directions, got: ', pbc_length(1:3) 
-            write(*,*) '        Lower PBC coordinates read: ', pbc_box(1:3)
-            write(*,*) '        Upper PBC coordinates read: ', pbc_box(4:6)
+         pbc_length = pbc_max - pbc_min
+         if ( any(pbc_length <= 0.0) ) then
+            write(*,*) ' ERROR: Length of PBC box must be positive in all directions, got: ', &
+              pbc_length
+            write(*,*) '        Lower PBC coordinates read: ', pbc_min
+            write(*,*) '        Upper PBC coordinates read: ', pbc_max
             stop exit_code_input_error
          end if
-         write(*,"(a, 3f8.2)") '  Periodic boundary condition activated: ', pbc_length(1), pbc_length(2), pbc_length(3)
-         write(*,"(a, 6f8.2)") '  PBC Reference box: ', pbc_box(2), pbc_box(2), pbc_box(3), pbc_box(4), pbc_box(5), pbc_box(6)
+         write(*,"(a)") '  Periodic boundary condition activated: '
+         write(*,"(a, 3f8.2)") "    Minimum coordinates: ", pbc_min
+         write(*,"(a, 3f8.2)") "    Maximum coordinates: ", pbc_max
       else if( keyword(i,1) /= 'tolerance' .and. &
          keyword(i,1) /= 'short_tol_dist' .and. &
          keyword(i,1) /= 'short_tol_scale' .and. &
@@ -211,7 +224,6 @@ subroutine getinp()
          keyword(i,1) /= 'chain' .and. &
          keyword(i,1) /= 'discale' .and. &
          keyword(i,1) /= 'maxit' .and. &
-         keyword(i,1) /= 'movebadrandom' .and. &
          keyword(i,1) /= 'maxmove' .and. &
          keyword(i,1) /= 'add_amber_ter' .and. &
          keyword(i,1) /= 'amber_ter_preserve' .and. &
@@ -223,6 +235,7 @@ subroutine getinp()
          keyword(i,1) /= 'nloop' .and. &
          keyword(i,1) /= 'nloop0' .and. &
          keyword(i,1) /= 'writeout' .and. &
+         keyword(i,1) /= 'hexadecimal_indices' .and. &
          keyword(i,1) /= 'writebad' .and. &
          keyword(i,1) /= 'check' .and. &
          keyword(i,1) /= 'iprint1' .and. &
@@ -290,6 +303,7 @@ subroutine getinp()
                   idatom = idatom + 1
                   amass(idatom) = 1.d0
                   maxcon(idatom) = 0
+                  nconnect(idatom,1:9) = 0
                   ! Read the index of the first atom, to adjust connectivities, if any
                   if(idfirstatom == 0) read(record(7:11),*,iostat=ioerr) idfirstatom
                   read(record,"( t31,f8.3,t39,f8.3,t47,f8.3 )",iostat=ioerr) &
@@ -332,31 +346,30 @@ subroutine getinp()
                if ( ioerr /= 0 ) exit
                if(record(1:6).eq.'CONECT') then
                   iread = 7
-                  read(record(iread:iread+4),*,iostat=ioerr) iatom
-                  iatom = iatom - idfirstatom + 1
-                  idatom = idfirst(itype) - 1 + iatom
+                  read(record(7:len(record)),*,iostat=ioerr) iatom
                   if(ioerr /= 0) then
                      write(*,*) " ERROR: Could not read atom index from CONECT line: "
                      write(*,*) trim(adjustl(record))
                      stop exit_code_input_error
                   end if
-                  iread = iread + 5
-                  read(record(iread:iread+4),*,iostat=ioerr) nconnect(idatom,1)
-                  if(ioerr /= 0) then
-                     write(*,*) " ERROR: Could not read any connection index from CONECT line: "
+                  iatom = iatom - idfirstatom + 1
+                  idatom = idfirst(itype) - 1 + iatom
+                  ! count the number of connections
+                  read(record(7:len(record)), *, iostat=ioerr)  iatom, &
+                      (nconnect(idatom, i), i=maxcon(idatom)+1, 9)
+                  if(ioerr > 0) then
+                     write(*,*) " ERROR: Error reading CONECT line: "
                      write(*,*) trim(adjustl(record))
                      stop exit_code_input_error
                   end if
-                  nconnect(idatom,1) = nconnect(idatom,1) - idfirstatom + 1
-                  maxcon(idatom) = 1
-                  do while(.true.)
-                     iread = iread + 5
-                     read(record(iread:iread+4),*,iostat=ioerr) nconnect(idatom,maxcon(idatom)+1)
-                     if(ioerr == 0) then
-                        maxcon(idatom) = maxcon(idatom) + 1
-                        nconnect(idatom,maxcon(idatom)) = nconnect(idatom,maxcon(idatom)) - idfirstatom + 1
-                     else
-                        exit
+                  do i = maxcon(idatom)+1, 9
+                     if (nconnect(idatom,i) /= 0) then
+                        nconnect(idatom, i) = nconnect(idatom, i) - idfirstatom + 1
+                     end if
+                  end do
+                  do i = 1, 9
+                     if (nconnect(idatom, i) /= 0) then
+                        maxcon(idatom) = i
                      end if
                   end do
                end if
@@ -366,7 +379,6 @@ subroutine getinp()
          end if
 
          ! Reading tinker input files
-
          if(tinker) then
             open(10,file=keyword(iline,2),status='old',iostat=ioerr)
             if ( ioerr /= 0 ) call failopen(keyword(iline,2))
@@ -453,7 +465,6 @@ subroutine getinp()
          end if
 
          ! Reading xyz input files
-
          if(xyz) then
             open(10,file=keyword(iline,2),status='old',iostat=ioerr)
             if ( ioerr /= 0 ) call failopen(keyword(iline,2))
@@ -475,35 +486,7 @@ subroutine getinp()
             close(10)
          end if
 
-         ! Reading moldy input files
 
-         if(moldy) then
-            open(10,file=keyword(iline,2), status ='old',iostat=ioerr)
-            if ( ioerr /= 0 ) call failopen(keyword(iline,2))
-            read(10,*) name(itype), nmols(itype)
-            natoms(itype) = 0
-            do while(.true.)
-               read(10,str_format,iostat=ioerr) record
-               if ( ioerr /= 0 ) exit
-               if(record.gt.' '.and.record(1:3).ne.'end') &
-                  natoms(itype) = natoms(itype) + 1
-            end do
-            close(10)
-            idfirst(itype) = 1
-            do ii = itype - 1, 1, -1
-               idfirst(itype) = idfirst(itype) + natoms(ii)
-            end do
-            open(10,file=keyword(iline,2),status='old')
-            read(10,str_format) record
-            idatom = idfirst(itype) - 1
-            do iatom = 1, natoms(itype)
-               idatom = idatom + 1
-               read(10,str_format) record
-               read(record,*) lixo, (coor(idatom,k), k = 1, 3),&
-                  amass(idatom), charge(idatom), ele(idatom)
-            end do
-            close(10)
-         end if
       end if
 
    end do
@@ -751,25 +734,6 @@ subroutine getinp()
 
    end do
 
-   if (using_pbc) then
-      irest = irest + 1
-      irestline(irest) = -1
-      ityperest(irest) = 3
-      restpars(irest,1) = pbc_box(1) - dism/2
-      restpars(irest,2) = pbc_box(2) - dism/2
-      restpars(irest,3) = pbc_box(3) - dism/2
-      restpars(irest,4) = pbc_box(4) + dism/2
-      restpars(irest,5) = pbc_box(5) + dism/2
-      restpars(irest,6) = pbc_box(6) + dism/2
-      write(*,*) " PBC on: We automatically add a constraint for non-fixed atoms:"
-      write(*,"(a, 6f8.2)") "  -> inside box ", restpars(irest,1), restpars(irest,2), restpars(irest,3),&
-         restpars(irest,4), restpars(irest,5), restpars(irest,6)
-   end if
-
-   nrest = irest
-   write(*,*) ' Total number of restrictions: ', nrest
-
-
    ! Reading, if defined, the short distance penalty parameters
 
    ioerr = 1
@@ -957,6 +921,10 @@ subroutine getinp()
       end if
    end do lines
 
+   ! total number of restrictions
+   nrest = irest
+   write(*,*) ' Total number of restrictions: ', nrest
+
    return
 end subroutine getinp
 
