@@ -16,7 +16,7 @@ module input
    integer :: seed
    integer :: nloop, nloop_all
    integer :: writeout
-   integer :: ntfix
+   integer :: ntype_with_fixed
    integer :: ntcon(9)
 
    integer, allocatable :: nconnect(:,:)  ! (ntotat,9)
@@ -48,7 +48,6 @@ module input
    logical :: pdb
    logical :: crd
    logical :: xyz
-   logical :: moldy
    logical :: check
    logical :: chkgrad
    logical :: randini
@@ -59,6 +58,8 @@ module input
    logical :: avoidoverlap
    logical :: packall
    logical :: use_short_tol
+   logical :: hexadecimal_indices
+   logical :: disable_movebad
 
    logical, allocatable :: changechains(:) ! (ntype)
    logical, allocatable :: fixedoninput(:) ! (ntype)
